@@ -1,6 +1,10 @@
 #!perl -Tw
 # Copyright Dominique Quatravaux 2006 - Licensed under the same terms as Perl itself
 
+use strict;
+use warnings;
+use 5.006; # "our" keyword
+
 =head1 NAME
 
 B<My::Tests::Below> - invoke a test suite at the end of a module.
@@ -243,7 +247,11 @@ is actually needed.
     $ENV{PATH} =~ m|^(.*)$|; # Untaints
     local %ENV = (
             "PATH"  => $1,
-            "DEBUG" => $ENV{"DEBUG"} ? 1 : 0,
+            ( $ENV{"DEBUG"} ? ( DEBUG => 1 ) : () ),
+            # Keep the following variables as some tools (eg ccache) freak
+            # out when they are missing altogether:
+            ( map { defined($ENV{$_}) ? ( $_ => $ENV{$_} ) : () }
+              qw(HOME USER LOGNAME) ),
            );
 
     eval $self->{testsuite};
