@@ -10,6 +10,8 @@ package Crypt::OpenSSL::CA::Inline::C;
 
 Crypt::OpenSSL::CA::Inline::C - A bag of XS and L<Inline::C> tricks
 
+=encoding utf8
+
 =head1 SYNOPSIS
 
 =for My::Tests::Below "synopsis" begin
@@ -85,12 +87,13 @@ the following C functions:
 
   static inline char* char0_value(SV* string);
 
-Returns the string value of a Perl SV, making sure that it exists and
-is zero-terminated beforehand. If C<string> is undef, returns the
-empty string (B<not> NULL; see L</char0_value_or_null>).  See
-L<perlguts/Working with SVs>, look for the word "Nevertheless" - I'm
-pretty sure there is a macro in Perl's convenience stuff that does
-exactly that already, but I don't know it...
+Returns the string value of a Perl SV, making sure that it exists and is
+zero-terminated beforehand. If C<string> is undef, returns the empty string
+(B<not> NULL; see L</char0_value_or_null>). If C<string> is tainted, that makes
+no difference; return it, or the empty string, just the same. See
+L<perlguts/Working with SVs>, look for the word "Nevertheless" - I assume
+there is a macro in Perl's convenience stuff that does exactly that already, but
+I don't know it...
 
 =head3 char0_value_or_null
 
@@ -262,12 +265,13 @@ sub _c_boilerplate { <<'C_BOILERPLATE'; }
 #endif
 
 static inline char* char0_value_or_null(SV* perlscalar) {
-     STRLEN length; char* retval;
+     if (! SvOK(perlscalar)) { return NULL; }
 
+     STRLEN length;
      SvPV(perlscalar, length);
-     if (! SvPOK(perlscalar)) { return NULL; }
      SvGROW(perlscalar, length + 1);
-     retval = SvPV_nolen(perlscalar);
+
+     char* retval = SvPV_nolen(perlscalar);
      retval[length] = '\0';
      return retval;
 }
@@ -375,7 +379,7 @@ static void sslcroak(char *fmt, ...) {
          croak(Nullch);
     } else {
          // Something went bang, revert to the croakbuf.
-         croak(croakbuf);
+         croak("%s", croakbuf);
     }
 }
 
@@ -422,8 +426,8 @@ static ASN1_TIME* parse_RFC3280_time_or_croak(char* date) {
     if ((retval = parse_RFC3280_time(date, &plainerr, &sslerr))) {
         return retval;
     }
-    if (plainerr) { croak(plainerr); }
-    if (sslerr) { sslcroak(sslerr); }
+    if (plainerr) { croak("%s", plainerr); }
+    if (sslerr) { sslcroak("%s", sslerr); }
     croak("Unknown error in parse_RFC3280_time");
     return NULL; /* Not reached */
 }
@@ -458,8 +462,8 @@ static ASN1_INTEGER* parse_serial_or_croak(char* hexserial) {
     if ((retval = parse_serial(hexserial, &plainerr, &sslerr))) {
         return retval;
     }
-    if (plainerr) { croak(plainerr); }
-    if (sslerr) { sslcroak(sslerr); }
+    if (plainerr) { croak("%s", plainerr); }
+    if (sslerr) { sslcroak("%s", sslerr); }
     croak("Unknown error in parse_serial");
     return NULL; /* Not reached */
 }
@@ -648,7 +652,7 @@ sub import {
     my ($package) = caller;
     no strict "refs";
     push @{$package."::ISA"}, qw(XSLoader);
-    XSLoader::load($package);
+    { no warnings "redefine"; XSLoader::load($package); }
 }
 
 =head1 NAME
@@ -730,7 +734,7 @@ __END__
 
 =cut
 
-use Test::More no_plan => 1;
+use Test::More "no_plan";
 use Test::Group;
 use Crypt::OpenSSL::CA::Test qw(errstack_empty_ok
                                 cannot_check_SV_leaks leaks_SVs_ok
